@@ -21,10 +21,14 @@ def millis():
 class Event(object):
 
 	def __call__(self, value=0):
+		if self.event is None:
+			self.event = self.sm.map_to_sim_event(self.deff)
 		self.sm.send_event(self.event, DWORD(value))
 
-	def __init__(self, _deff, _sm):
-		self.event = _sm.map_to_sim_event(_deff)
+	def __init__(self, _deff, _sm, _dec=''):
+		self.deff = _deff
+		self.event = None
+		self.description = _dec
 		self.sm = _sm
 
 
@@ -32,57 +36,121 @@ class Request(object):
 
 	@property
 	def value(self):
-		if (self.LastData + self.time) < millis():
-			# LOGGER.info("get Request:" + str(self.definitions[0]))
-			self.sm.get_data(self)
-			self.LastData = millis()
-		return self.outData[self.name]
+		if self._deff_test():
+			self.sm.run()
+			if (self.LastData + self.time) < millis():
+				if self.sm.get_data(self):
+					self.LastData = millis()
+				else:
+					return -999999
+			return self.outData[self.name]
+		else:
+			return None
 
 	@value.setter
 	def value(self, val):
-		self.outData[self.name] = val
-		self.sm.set_data(self)
+		if self._deff_test():
+			self.outData[self.name] = val
+			self.sm.set_data(self)
+			self.sm.run()
 
-	def __init__(self, _deff, _sm, _time=2000):
-		(_DATA_DEFINITION_ID, _DATA_REQUEST_ID) = _sm.new_request_id()
-		self.DATA_DEFINITION_ID = _DATA_DEFINITION_ID
-		self.DATA_REQUEST_ID = _DATA_REQUEST_ID
+	def __init__(self, _deff, _sm, _time=2000, _dec=None):
+		self.DATA_DEFINITION_ID = None
 		self.definitions = []
+		self.description = _dec
 		self.definitions.append(_deff)
 		self.name = "_name"
-		self.outData = {}
+		self.outData = {self.name: 0}
 		self.sm = _sm
 		self.time = _time
+		self.defined = False
 		self.LastData = 0
-		self.outData[self.name] = len(self.outData)
-		_sm.out_data[self.DATA_REQUEST_ID] = None
-		_sm.Requests.append(self)
-		_sm.dll.AddToDataDefinition(
-			_sm.hSimConnect,
+		if ':index' in str(self.definitions[0][0]):
+			self.lastIndex = b':index'
+
+	def setIndex(self, index):
+		(dec, stype) = self.definitions[0]
+		newindex = str(":" + str(index)).encode()
+		dec = dec.replace(self.lastIndex, newindex)
+		self.lastIndex = newindex
+		self.definitions[0] = (dec, stype)
+		self.redefine()
+
+	def redefine(self):
+		if self.DATA_DEFINITION_ID is not None:
+			self.sm.dll.ClearDataDefinition(
+				self.sm.hSimConnect,
+				self.DATA_DEFINITION_ID.value,
+			)
+			self.defined = False
+			self.sm.run()
+		if self._deff_test():
+			self.sm.run()
+			self.sm.get_data(self)
+
+	def _deff_test(self):
+		if ':index' in str(self.definitions[0][0]):
+			self.lastIndex = b':index'
+			return False
+		if self.defined is True:
+			return True
+		if self.DATA_DEFINITION_ID is None:
+			(self.DATA_DEFINITION_ID, self.DATA_REQUEST_ID) = self.sm.new_request_id()
+			self.sm.out_data[self.DATA_REQUEST_ID] = None
+			self.sm.Requests.append(self)
+
+		err = self.sm.dll.AddToDataDefinition(
+			self.sm.hSimConnect,
 			self.DATA_DEFINITION_ID.value,
-			_deff[0],
-			_deff[1],
+			self.definitions[0][0],
+			self.definitions[0][1],
 			SIMCONNECT_DATATYPE.SIMCONNECT_DATATYPE_FLOAT64,
 			0,
 			SIMCONNECT_UNUSED,
 		)
+		if IsHR(err, 0):
+			self.defined = True
+			return True
+		else:
+			LOGGER.error("SIM def" + str(self.definitions[0]))
+			return False
 
 
 class requestHolder:
 	def __init__(self, _sm, _time=2000):
 		self._sm = _sm
 		self._time = _time
+		self.dic = []
 
-	def add(self, name, _deff):
+	def json(self):
+		map = {}
+		for att in self.dic:
+			val = self.get(att)
+			if val is not None:
+				map[att] = val
+		return map
+
+	def add(self, name, _deff, _dec=''):
+		self.dic.append(name)
 		setattr(
 			self,
 			name,
-			Request(_deff, self._sm, self._time)
+			Request(_deff, self._sm, self._time, _dec)
 		)
 
-	def get(self, _name):
+	def obj(self, _name):
+		return getattr(self, _name)
+
+	def get(self, _name, _dec=False):
+		if _dec is True and getattr(self, _name).description is not None:
+			return (getattr(self, _name).value, getattr(self, _name).description)
 		return getattr(self, _name).value
 
+	def set(self, _name, _value):
+		temp = getattr(self, _name)
+		setattr(temp, "value", _value)
+
+
 class SimConnect:
 
 	# TODO: update callbackfunction to expand functions.
@@ -92,7 +160,7 @@ class SimConnect:
 		if dwID == SIMCONNECT_RECV_ID.SIMCONNECT_RECV_ID_EVENT:
 			evt = cast(pData, POINTER(SIMCONNECT_RECV_EVENT))
 			uEventID = evt.contents.uEventID
-			if uEventID == self.EventID.EVENT_SIM_START:
+			if uEventID == self.dll.EventID.EVENT_SIM_START:
 				LOGGER.info("SIM START")
 
 		elif dwID == SIMCONNECT_RECV_ID.SIMCONNECT_RECV_ID_SIMOBJECT_DATA_BYTYPE:
@@ -107,8 +175,9 @@ class SimConnect:
 					).contents
 		elif dwID == SIMCONNECT_RECV_ID.SIMCONNECT_RECV_ID_OPEN:
 			LOGGER.info("SIM OPEN")
+			self.ok = True
 		elif dwID == SIMCONNECT_RECV_ID.SIMCONNECT_RECV_ID_EXCEPTION:
-			LOGGER.warn("ID EXCEPTION")
+			pass  # LOGGER.warn("ID EXCEPTION")
 		elif dwID == SIMCONNECT_RECV_ID.SIMCONNECT_RECV_ID_QUIT:
 			self.quit = 1
 		else:
@@ -122,6 +191,7 @@ class SimConnect:
 		self.dll = SimConnectDll(library_path)
 		self.hSimConnect = HANDLE()
 		self.quit = 0
+		self.ok = False
 		self.my_dispatch_proc_rd = self.dll.DispatchProc(self.my_dispatch_proc)
 		if auto_connect:
 			self.connect()
@@ -138,11 +208,14 @@ class SimConnect:
 				self.dll.SubscribeToSystemEvent(
 					self.hSimConnect, self.dll.EventID.EVENT_SIM_START, b"SimStart"
 				)
+				while self.ok is False:
+					self.run()
 		except OSError:
 			LOGGER.debug("Did not find Flight Simulator running.")
 			exit(0)
 
 	def run(self):
+		time.sleep(.01)
 		self.dll.CallDispatch(self.hSimConnect, self.my_dispatch_proc_rd, None)
 
 	def exit(self):
@@ -186,7 +259,7 @@ class SimConnect:
 		pObjData = cast(
 			dataarray, c_void_p
 		)
-		self.dll.SetDataOnSimObject(
+		err = self.dll.SetDataOnSimObject(
 			self.hSimConnect,
 			_Request.DATA_DEFINITION_ID.value,
 			SIMCONNECT_SIMOBJECT_TYPE.SIMCONNECT_SIMOBJECT_TYPE_USER,
@@ -195,12 +268,22 @@ class SimConnect:
 			sizeof(ctypes.c_double) * len(pyarr),
 			pObjData
 		)
+		if IsHR(err, 0):
+			# LOGGER.debug("Request Sent")
+			return True
+		else:
+			return False
 
 	def get_data(self, _Request):
 		self.request_data(_Request)
 		self.run()
-		while self.out_data[_Request.DATA_REQUEST_ID] is None:
+		attemps = 0
+		while self.out_data[_Request.DATA_REQUEST_ID] is None and attemps < 4:
 			self.run()
+			attemps += 1
+
+		if self.out_data[_Request.DATA_REQUEST_ID] is None:
+			return False
 
 		newData = self.out_data[_Request.DATA_REQUEST_ID]
 		for od in _Request.outData:
@@ -219,12 +302,12 @@ class SimConnect:
 			DWORD(16),
 		)
 		if IsHR(err, 0):
-			LOGGER.debug("Event Sent")
+			# LOGGER.debug("Event Sent")
 			return True
 		else:
 			return False
 
-	def new_request_id(self, time=None):
+	def new_request_id(self):
 		name = "Request" + str(len(self.Requests))
 		names = [m.name for m in self.dll.DATA_DEFINITION_ID] + [name]
 		self.dll.DATA_DEFINITION_ID = Enum(self.dll.DATA_DEFINITION_ID.__name__, names)
